@JS()
library jsApi;

import 'dart:async';
import 'dart:convert';

import "package:js/js.dart";
import 'package:http/browser_client.dart';
import 'package:http/http.dart';

@JS('top')
abstract class Top {
  @JS('injectJsApi')
  external static void injectJsApi(Top top, Window window);
}

@JS('window')
abstract class Window {}

@JS("Promise")
class Promise {
  external void then(Function onFulfilled, Function onRejected);

  external static Promise resolve(dynamic value);
}

@JS('jsApi')
abstract class JsApi {
  @JS('findContentCode')
  external static String findContentCode();

  @JS('extractSubjectUuid')
  external static String extractSubjectUuid();

  @JS('registerAttributeToModification')
  external static void registerAttributeToModification(
      String attrCode, Function resultCallBack);

  @JS('isAddForm')
  external static bool isAddForm();

  @JS('isEditForm')
  external static bool isEditForm();

  @JS('isOnObjectCard')
  external static bool isOnObjectCard();

  @JS('getCurrentUser')
  external static dynamic getCurrentUser();

  @JS('getAppBaseUrl')
  external static String getAppBaseUrl();

  @JS('getAppRestBaseUrl')
  external static String getAppRestBaseUrl();

  @JS('restCall')
  external static Promise restCall(String restOfTheUrl, Map options);

  @JS('restCallAsJson')
  external static Promise restCallAsJson(String restOfTheUrl, Map options);

  @JS('addFieldChangeListener')
  external static void addFieldChangeListener(
      String attrCode, void Function(Attribute) callback);

  @JS('commands')
  external static Commands get commands;

  @JS('requests')
  external static Requests get requests;

  @JS('urls')
  external static Urls get urls;

  @JS('configuration')
  external static Configuration get configuration;
}

@JS()
@anonymous
abstract class Commands {
  @JS('getCurrentContextObject')
  external Promise getCurrentContextObject();

  @JS('selectObjectDialog')
  external Promise selectObjectDialog(
      String classFqn, String presentAttributesGroupCode);

  @JS('changeState')
  external void changeState(String uuid, List<String> states);

  @JS('editObject')
  external Promise editObject(String uuid);
}

@JS()
@anonymous
abstract class Requests {
  @JS('make')
  external static Promise make(Map options);

  @JS('json')
  external static Promise json(Map options);
}

@JS()
@anonymous
abstract class Urls {
  @JS('base')
  external static String base();

  @JS('objectCard')
  external static String objectCard(String uuid);

  @JS('objectEditForm')
  external static String objectEditForm(String uuid);

  @JS('objectAddForm')
  external static String objectAddForm(String fqn);
}

@JS()
@anonymous
abstract class Configuration {
  @JS('byContentCode')
  external static Promise byContentCode(String moduleCode, List args);

  @JS('byDefault')
  external static Promise byDefault(String moduleCode, List args);
}

@JS()
@anonymous
abstract class SmpResponse {}

@JS()
@anonymous
abstract class Attribute {
  external String get attribute;

  external dynamic get newValue;
}

@JS()
@anonymous
class StringAttribute extends Attribute {
  external String get attribute;

  external String get newValue;
}

@JS()
@anonymous
class StringList extends Attribute {
  external String get attribute;

  external List<String> get newValue;
}

@JS()
@anonymous
class BOLink extends Attribute {
  external String get attribute;

  external BObject get newValue;
}

@JS()
@anonymous
class BOLinks extends Attribute {
  external String get attribute;

  external List<BObject> get newValue;
}

@JS()
@anonymous
class BObject {
  external String get title;

  external String get UUID;
}

class SmpAPI {
  /// Таймаут на поиск родительского окна
  static final int _timeOut = 1000;

  /// Заголовки для REST API запросов в сторону сервера Naumen SMP
  static final _headers = {'Content-Type': 'application/json'};

  /// Адрес REST API сервиса Naumen SMP для поиска объектов
  static String _find = '../services/rest/find';

  /// Адрес REST API сервиса Naumen SMP для получения объекта по UUID
  static String _get = '../services/rest/get';

  /// Адрес REST API сервиса Naumen SMP для редактирования объекта
  static String _edit = '../services/rest/edit';

  /// Адрес REST API сервиса Naumen SMP для создания объекта M2M
  static String _create = '../services/rest/create-m2m';

  /// Адрес REST API сервиса Naumen SMP для исполнения скриптовых модулей или удаленного выполнения скриптов
  static String _execPost = '../services/rest/exec-post';

  static BrowserClient _http = new BrowserClient();
  static JsonCodec json = const JsonCodec();

  static String get apiType => 'Dart Naumen SMP API';

  /// Метод REST API сервиса Naumen SMP для поиска, возвращает первый найденный
  static Future<Map> findFirst(String url) async {
    try {
      List data = await find(url);
      return data.length > 0 ? data.first : {};
    } catch (e) {
      print(e.toString());
      return {};
    }
  }

  /// Метод REST API сервиса Naumen SMP для получения объекта по URL
  static Future<Map> getObjectByUrl(String url) async {
    try {
      final response = await _http.get(url);
      return _extractData(response);
    } catch (e) {
      return {};
    }
  }

  /// Метод REST API сервиса Naumen SMP для получения объекта по UUID
  static Future<Map> get(String url) async {
    try {
      final response = await _http.get('$_get/$url');
      return _extractData(response);
    } catch (e) {
      return {};
    }
  }

  /// Метод REST API сервиса Naumen SMP для поиска объектов
  static Future<List> find(String url) async {
    try {
      final response = await _http.get('$_find$url');
      return _extractData(response);
    } catch (e) {
      return [];
    }
  }

  /// Метод REST API сервиса Naumen SMP для создания объекта
  static Future<Map> create(String url, Map data) async {
    try {
      final response = await _http.post('$_create$url',
          headers: _headers, body: json.encode(data));
      return _extractData(response);
    } catch (e) {
      print(e.toString());
      return {};
    }
  }

  /// Метод REST API сервиса Naumen SMP для исполнения скриптового модуля с передачей requestContent
  static Future<Map<String, dynamic>> execPost(String url, Map data) async {
    try {
      final response = await _http.post('$_execPost$url',
          headers: _headers, body: json.encode(data));
      return _extractData(response);
    } catch (e) {
      print(e.toString());
      return {};
    }
  }

  /// Метод REST API сервиса Naumen SMP для редактирования объекта
  static Future<String> edit(String url, Map data) async {
    String body = json.encode(data);
    try {
      final response =
          await _http.post('$_edit$url', headers: _headers, body: body);
      return response.body;
    } catch (e) {
      print(e.toString());
      return 'Rest error';
    }
  }

  /// Метод REST API сервиса Naumen SMP для получения объекта по URL
  static _extractData(Response resp) => json.decode(resp.body);
}
